@@ -1,6 +1,6 @@
 import ms2ml.data.adapters as adapters
 import ms2ml.data.parsing as parsing
 
-from .utils import default_collate, hook_collate, pad_to_shape
+from ..utils.tensor_utils import default_collate, hook_collate, pad_to_shape
 
 __all__ = ["default_collate", "hook_collate", "pad_to_shape", "adapters", "parsing"]
